<?php

namespace Codemanas\Typesense\Elementor\Widgets;

if( ! defined('ABSPATH')){
	exit;
}

class AutoComplete extends \Elementor\Widget_Base {

	public function __construct( $data = [], $args = null ) {
		parent::__construct( $data, $args );
	}

	public function get_script_depends() {
		return ['cm-typesense-autocomplete'];
	}

	public function get_name() {
		return 'cm-typesense-autocomplete';

	}

	public function get_title() {
		return esc_html( 'AutoComplete', 'search-with-typesense' );
	}


	public function get_icon() {
		return 'eicon-search-bold';
	}

	public function get_custom_help_url() {
		return 'https://codemanas.github.io/cm-typesense-docs/shortcode/#autocomplete';
	}

	public function get_categories() {
		return [ 'cm-typesense' ];
	}

	public function get_keywords() {
		return [ 'autocomplete', 'cm', 'typesense' ];
	}

	protected function register_controls() {
		parent::register_controls(); // TODO: Change the autogenerated stub
		$this->start_controls_section(
			'content_section',
			[
				'label' => esc_html__( 'Settings', 'search-with-typesense' ),
				'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		$this->add_responsive_control(
			'search_width',
			[
				'label' => esc_html__( 'Search box width', 'search-with-typesense' ),
				'type' => \Elementor\Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 5,
					],
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => '',
				],
				'selectors' => [
					'{{WRAPPER}} .cm-autocomplete' => 'width: {{SIZE}}{{UNIT}};',
				],
			],

		);
		$this->add_control(
			'place_holder',
			[
				'type' => \Elementor\Controls_Manager::TEXT,
				'label' => esc_html__( 'Placeholder', 'search-with-typesense' ),
				'default' => esc_html( 'Search for:', 'search-with-typesense' ),
			]
		);
		$this->add_control(
			'query_by',
			[
				'type' => \Elementor\Controls_Manager::TEXT,
				'label' => esc_html__( 'Query by', 'search-with-typesense' ),
				
			]
		);
	}

	protected function render() {
		$settings = $this->get_settings_for_display();

		$place_holder = $settings['place_holder'];
		$query_by = $settings['query_by'];
		echo do_shortcode('[cm_typesense_autocomplete placeholder="'.$place_holder.'" query_by="'. $query_by .'"]');
	}
}